@@ -19,7 +19,7 @@
 #   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 
-push @VERSION, '$Id: storeBackupLib.pl 327 2004-03-07 17:44:44Z hjc $ ';
+push @VERSION, '$Id: storeBackupLib.pl 335 2004-07-23 03:56:18Z hjc $ ';
 
 use strict;
 
@@ -36,11 +36,13 @@ sub buildDBMs
 {
     my (%params) = ('-dbmKeyIsFilename'    => undef,     # Hash
 		    '-dbmKeyIsMD5Sum'      => undef,     # Hash
+		    '-indexDir'            => undef,     # object pointer
 		    '-previousDir'         => undef,     # dbm{filename} wird nur
 		                                         # gefllt, wenn -backupRoot
 		                                         # eq -previousDir
 		    '-backupRoot'          => undef,     # String
 		    '-backupDirIndex'      => undef,     # Index des Pfades
+		    '-noBackupDir'         => undef,
 		    '-checkSumFile'        => undef,
 		    '-checkSumFileVersion' => undef,
 		    '-progressReport'      => undef,
@@ -48,20 +50,23 @@ sub buildDBMs
 		    );
 
     &::checkObjectParams(\%params, \@_, '::buildDBMs',
-			 ['-dbmKeyIsFilename', '-dbmKeyIsMD5Sum', '-previousDir',
-			  '-backupRoot', '-backupDirIndex', '-checkSumFile',
+			 ['-dbmKeyIsFilename', '-dbmKeyIsMD5Sum', '-indexDir',
+			  '-previousDir', '-backupRoot', '-backupDirIndex',
+			  '-noBackupDir', '-checkSumFile',
 			  '-checkSumFileVersion', '-prLog']);
     my $dbmKeyIsFilename = $params{'-dbmKeyIsFilename'};
     my $dbmKeyIsMD5Sum = $params{'-dbmKeyIsMD5Sum'};
+    my $indexDir = $params{'-indexDir'};
     my $previousDir = $params{'-previousDir'};
     my $backupRoot = $params{'-backupRoot'};
     my $backupDirIndex = $params{'-backupDirIndex'};
+    my $noBackupDir = $params{'-noBackupDir'};
     my $checkSumFile = $params{'-checkSumFile'};
     my $checkSumFileVersion = $params{'-checkSumFileVersion'};
     my $progressReport = 5 * $params{'-progressReport'};
     my $prLog = $params{'-prLog'};
 
-    my $storeDBMfilename = ($previousDir eq $backupRoot) ? 1 : 0;
+    my $isPreviousBackup = ($previousDir eq $backupRoot) ? 1 : 0;
 
     my $rcsf = readCheckSumFile->new('-checkSumFile' =>
 				     "$backupRoot/$checkSumFile",
@@ -93,29 +98,33 @@ sub buildDBMs
 
 	next if (length($md5sum) != 32);  # ist dir, pipe, symlink
 
-	$$dbmKeyIsFilename{$f} = pack('aJJJH32', $compr, $ctime, $mtime,
-				      $size, $md5sum);
+	my ($fbase, $fname, $index) = $indexDir->newFile($f);
+
+	$$dbmKeyIsFilename{"$index/$fname"} =
+	    pack('aIIIH32', $compr, $ctime, $mtime, $size, $md5sum)
+	    if ($isPreviousBackup or not
+		exists $$dbmKeyIsFilename{"$index/$fname"});
 
 	my $md5pack = pack('H32', $md5sum);
-	if (exists $$dbmKeyIsMD5Sum{$md5pack})
+	if ($noBackupDir > 1 and exists $$dbmKeyIsMD5Sum{$md5pack})
 	{
-	    my ($_inodeBackup) = unpack('J', $$dbmKeyIsMD5Sum{$md5pack});
+	    my ($_inodeBackup) = unpack('I', $$dbmKeyIsMD5Sum{$md5pack});
 	    next if ($_inodeBackup <= $inodeBackup);
 	}
 	else
 	{
 	    ++$noEntriesInDBM;
 	}
-
 	# gibt's noch nicht bzw. $inodeBackup ist kleiner als in dbm file
-	$$dbmKeyIsMD5Sum{$md5pack} = pack('JaSa*', $inodeBackup, $compr,
-					 $backupDirIndex, $f);
+	$$dbmKeyIsMD5Sum{$md5pack} = pack('IaSa*', $inodeBackup, $compr,
+					 $backupDirIndex, "$index/$fname");
     }
 
     $prLog->print('-kind' => 'I',
 		  '-str' =>
 		  ["finished reading " . $rcsf->getFilename() .
 		   " ($noLines entries)"]);
+
     return $noEntriesInDBM;
 }
 
@@ -1918,6 +1927,7 @@ sub new
     my (%args) = ('tmpDir' => $params{'-tmpdir'},
 		  'otherBackupDirs' => undef,
 		  'lockFile' => undef,
+		  'unlockBeforeDel' => 'no',
 		  'exceptDirs' => undef,
 		  'includeDirs' => undef,
 		  'contExceptDirsErr' => 'no',
@@ -2051,7 +2061,8 @@ sub new
 		unless ($l =~ /\A[Sbcfpl]*\Z/);
 	    $args{$key} = $l if ($l ne '');
 	}
-	elsif ($key eq 'contExceptDirsErr' or   # 'yes' oder 'no'
+	elsif ($key eq 'unlockBeforeDel' or      # 'yes' oder 'no'
+	       $key eq 'contExceptDirsErr' or
 	       $key eq 'compressMD5File' or
 	       $key eq 'verbose' or
 	       $key eq 'resetAtime' or
@@ -2218,4 +2229,77 @@ sub print
 }
 
 
+##################################################
+# generates an index out of a directory name
+# requests: 'index -> dir' or 'dir -> index'
+# this is for shorten the berkely db files and
+# therefor for better caching of it
+package indexDir;
+
+sub new
+{
+    my $class = shift;
+    my $self = {};
+
+    my (%indexToDir) = ();
+    my (%dirToIndex) = ();
+    $self->{'indexToDir'} = \%indexToDir;
+    $self->{'dirToIndex'} = \%dirToIndex;
+
+    $self->{'count'} = 0;
+
+    bless $self, $class;
+}
+
+
+########################################
+sub newFile
+{
+    my $self = shift;
+
+    my $file = shift;
+
+    my ($d, $f) = &::splitFileDir($file);
+
+    my $dirToIndex = $self->{'dirToIndex'};
+    if (exists($$dirToIndex{$d}))
+    {
+	return ($d, $f, $$dirToIndex{$d});
+    }
+    else
+    {
+	my $indexToDir = $self->{'indexToDir'};
+	$$dirToIndex{$d} = $self->{'count'};
+	$$indexToDir{$self->{'count'}} = $d;
+
+	return ($d, $f, $self->{'count'}++);
+    }
+}
+
+
+########################################
+sub replaceIndex
+{
+    my $self = shift;
+
+    my $fileWithIndex = shift;
+
+    my ($index, $f) = split('/', $fileWithIndex, 2);
+    my $indexToDir = $self->{'indexToDir'};
+    return $$indexToDir{$index} . "/$f";
+}
+
+
+########################################
+sub setIndex
+{
+    my $self = shift;
+
+    my $fileWithoutIndex = shift;
+
+    my ($d, $f, $index) = $self->newFile($fileWithoutIndex);
+    return "$index/$f";
+}
+
+
 1
